(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@ericblade/quagga2'), require('rxjs')) :
    typeof define === 'function' && define.amd ? define('@eisberg-labs/ngx-barcode-scanner', ['exports', '@angular/core', '@ericblade/quagga2', 'rxjs'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global['eisberg-labs'] = global['eisberg-labs'] || {}, global['eisberg-labs']['ngx-barcode-scanner'] = {}), global.ng.core, global.Quagga, global.rxjs));
}(this, (function (exports, i0, Quagga, rxjs) { 'use strict';

    function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

    function _interopNamespace(e) {
        if (e && e.__esModule) return e;
        var n = Object.create(null);
        if (e) {
            Object.keys(e).forEach(function (k) {
                if (k !== 'default') {
                    var d = Object.getOwnPropertyDescriptor(e, k);
                    Object.defineProperty(n, k, d.get ? d : {
                        enumerable: true,
                        get: function () {
                            return e[k];
                        }
                    });
                }
            });
        }
        n['default'] = e;
        return Object.freeze(n);
    }

    var i0__namespace = /*#__PURE__*/_interopNamespace(i0);
    var Quagga__default = /*#__PURE__*/_interopDefaultLegacy(Quagga);

    var NgxBarcodeScannerService = /** @class */ (function () {
        function NgxBarcodeScannerService() {
        }
        NgxBarcodeScannerService.prototype.defaultConfig = function () {
            return {
                inputStream: {
                    name: 'Live',
                    type: 'LiveStream',
                },
                locator: {
                    patchSize: 'medium',
                    halfSample: false
                },
                locate: true,
                numOfWorkers: 8,
                frequency: 10
            };
        };
        NgxBarcodeScannerService.prototype.meanBy = function (arr, property) {
            if (!arr) {
                return undefined;
            }
            return arr.reduce(function (acc, item) { return (property in item) ? acc + item[property] : acc; }, 0) / arr.length;
        };
        NgxBarcodeScannerService.prototype.isScanMatch = function (scanResult, errorThresholdPercentage) {
            var avgErrors = this.meanBy(scanResult.codeResult.decodedCodes, 'error');
            return avgErrors < errorThresholdPercentage;
        };
        NgxBarcodeScannerService.prototype.start = function (config, errorThresholdPercentage) {
            var _this = this;
            if (typeof this.scanResult === 'undefined') {
                this.scanResult = new rxjs.Subject();
            }
            Quagga__default['default'].onProcessed(function (scanResult) {
                _this.onProcessed(scanResult);
            });
            Quagga__default['default'].onDetected(function (result) {
                var barcode = result.codeResult.code;
                if (_this.isScanMatch(result, errorThresholdPercentage)) {
                    _this.scanResult.next(barcode + '');
                }
            });
            Quagga__default['default'].init(config, function (error) {
                if (error) {
                    _this.scanResult.error(error);
                    _this.stop();
                }
                else {
                    Quagga__default['default'].start();
                }
            });
            return this.scanResult;
        };
        NgxBarcodeScannerService.prototype.stop = function () {
            if (typeof this.scanResult !== 'undefined') {
                this.scanResult.unsubscribe();
            }
            Quagga__default['default'].stop();
        };
        NgxBarcodeScannerService.prototype.onProcessed = function (result) {
            var drawingCtx = Quagga__default['default'].canvas.ctx.overlay;
            var drawingCanvas = Quagga__default['default'].canvas.dom.overlay;
            if (result) {
                if (result.boxes) {
                    drawingCtx.clearRect(0, 0, parseInt(drawingCanvas.getAttribute('width'), 10), parseInt(drawingCanvas.getAttribute('height'), 10));
                    result.boxes.filter(function (box) { return box !== result.box; }).forEach(function (box) {
                        Quagga__default['default'].ImageDebug.drawPath(box, { x: 0, y: 1 }, drawingCtx, { color: 'green', lineWidth: 2 });
                    });
                }
                if (result.box) {
                    Quagga__default['default'].ImageDebug.drawPath(result.box, { x: 0, y: 1 }, drawingCtx, { color: '#00F', lineWidth: 2 });
                }
                if (result.codeResult && result.codeResult.code) {
                    Quagga__default['default'].ImageDebug.drawPath(result.line, { x: 'x', y: 'y' }, drawingCtx, { color: 'red', lineWidth: 3 });
                }
            }
        };
        return NgxBarcodeScannerService;
    }());
    NgxBarcodeScannerService.ɵprov = i0__namespace.ɵɵdefineInjectable({ factory: function NgxBarcodeScannerService_Factory() { return new NgxBarcodeScannerService(); }, token: NgxBarcodeScannerService, providedIn: "root" });
    NgxBarcodeScannerService.decorators = [
        { type: i0.Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    NgxBarcodeScannerService.ctorParameters = function () { return []; };

    var Utils = /** @class */ (function () {
        function Utils() {
        }
        Utils.setOrDefault = function (object, path, value) {
            if (typeof object[path] === 'undefined') {
                object[path] = value;
            }
        };
        return Utils;
    }());

    var NgxBarcodeScannerComponent = /** @class */ (function () {
        function NgxBarcodeScannerComponent(service) {
            this.service = service;
            this.codes = [
                'code_128', 'ean', 'ean_8', 'code_39', 'code_39_vin',
                'codabar', 'upc', 'upc_e', 'i2of5', '2of5', 'code_93'
            ];
            this.valueChange = new i0.EventEmitter();
            this.exception = new i0.EventEmitter();
        }
        NgxBarcodeScannerComponent.prototype.setConfig = function () {
            if (!this.config) {
                this.config = Object.assign(Object.assign({}, this.service.defaultConfig()), { decoder: {
                        readers: this.readers()
                    } });
            }
            if (!this.config.inputStream) {
                this.config.inputStream = {};
            }
            Utils.setOrDefault(this.config.inputStream, 'name', 'Live');
            Utils.setOrDefault(this.config.inputStream, 'type', 'LiveStream');
            if (!this.config.locator) {
                this.config.locator = {};
            }
            Utils.setOrDefault(this.config.locator, 'patchSize', 'medium');
            Utils.setOrDefault(this.config.locator, 'halfSample', false);
            Utils.setOrDefault(this.config, 'locate', true);
            Utils.setOrDefault(this.config, 'numOfWorkers', 8);
            Utils.setOrDefault(this.config, 'frequency', 10);
            if (!this.config.decoder) {
                this.config.decoder = {};
            }
            Utils.setOrDefault(this.config.decoder, 'readers', this.readers());
        };
        NgxBarcodeScannerComponent.prototype.ngOnInit = function () {
            var _this = this;
            this.setConfig();
            var threshold = isNaN(this.errorThreshold) ? 0.1 : this.errorThreshold;
            this.service.start(this.config, threshold).subscribe(function (value) {
                _this.valueChange.emit(value);
                _this.service.stop();
            }, function (error) {
                _this.exception.emit(error);
            });
        };
        NgxBarcodeScannerComponent.prototype.ngOnDestroy = function () {
            this.service.stop();
        };
        NgxBarcodeScannerComponent.prototype.readers = function () {
            var types = (typeof this.codes === 'string') ? [this.codes] : this.codes;
            return types.map(function (it) { return it + '_reader'; });
        };
        return NgxBarcodeScannerComponent;
    }());
    NgxBarcodeScannerComponent.decorators = [
        { type: i0.Component, args: [{
                    // tslint:disable-next-line:component-selector
                    selector: 'ngx-barcode-scanner',
                    template: "<div id=\"interactive\" class=\"viewport\"></div>\n",
                    styles: ["::ng-deep #interactive.viewport{position:relative;width:100%;height:auto;overflow:hidden;text-align:center}::ng-deep #interactive.viewport>canvas,::ng-deep #interactive.viewport>video{max-width:100%;width:100%}::ng-deep canvas.drawing,::ng-deep canvas.drawingBuffer{position:absolute;left:0;top:0}::ng-deep video{width:100%;-o-object-fit:cover;object-fit:cover}:host{position:static;width:100%}"]
                },] }
    ];
    NgxBarcodeScannerComponent.ctorParameters = function () { return [
        { type: NgxBarcodeScannerService }
    ]; };
    NgxBarcodeScannerComponent.propDecorators = {
        codes: [{ type: i0.Input }],
        config: [{ type: i0.Input }],
        errorThreshold: [{ type: i0.Input }],
        value: [{ type: i0.Input }],
        valueChange: [{ type: i0.Output }],
        exception: [{ type: i0.Output }]
    };

    var NgxBarcodeScannerModule = /** @class */ (function () {
        function NgxBarcodeScannerModule() {
        }
        return NgxBarcodeScannerModule;
    }());
    NgxBarcodeScannerModule.decorators = [
        { type: i0.NgModule, args: [{
                    declarations: [NgxBarcodeScannerComponent],
                    imports: [],
                    exports: [NgxBarcodeScannerComponent]
                },] }
    ];

    /*
     * Public API Surface of ngx-barcode-scanner
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.NgxBarcodeScannerComponent = NgxBarcodeScannerComponent;
    exports.NgxBarcodeScannerModule = NgxBarcodeScannerModule;
    exports.NgxBarcodeScannerService = NgxBarcodeScannerService;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=eisberg-labs-ngx-barcode-scanner.umd.js.map
